#include <SoftwareSerial.h>

SoftwareSerial BTSerial(2, 3);  //RX, TX
const int ledPin = 7;           
const int photoresistorPin = A0;  
const int threshold = 500;       

char buffer[100];           
int bufferIndex = 0;              
volatile bool readingFlag = false; 

char* letters[] = {
".-", "-...", "-.-.", "-..", ".", "..-.", "--.", "....", "..", // A-I
".---", "-.-", ".-..", "--", "-.", "---", ".--.", "--.-", ".-.", // J-R 
"...", "-", "..-", "...-", ".--", "-..-", "-.--", "--.." // S-Z
};
char* numbers[] = {
  "-----", ".----", "..---", "...--", "....-", ".....",
"-....", "--...", "---..", "----."
};
void setup() {
  pinMode(ledPin, OUTPUT);
  pinMode(photoresistorPin, INPUT);
  Serial.begin(9600);
  BTSerial.begin(9600);
  attachInterrupt(digitalPinToInterrupt(ledPin), startReading, CHANGE);
}

void loop() {
  if (Serial.available()) {
    String word = Serial.readStringUntil('\n');
    encryptToMorseCode(word);
    String decoded = decodeMorseCode(buffer);
    BTSerial.println("Decoded: " + decoded);
    printBuffer();
    resetBuffer();
    delay(2000);
  }
}


void encryptToMorseCode(String word) {
  for (int i = 0; i < word.length(); i++) {
    char ch = word.charAt(i);
    if (ch >= 'A' && ch <= 'Z') {
      ch += 32; 
    }

    if (ch >= 'a' && ch <= 'z') {
      char* morseCode = letters[ch - 'a'];
      for (int j = 0; morseCode[j] != '\0'; j++) {
        buffer[bufferIndex] = morseCode[j];
        bufferIndex++;
        blinkMorseCode(morseCode[j]);
      }
      buffer[bufferIndex] = ' ';  
      bufferIndex++;
      delay(300);  
    }
    else if (ch == ' ') {
      buffer[bufferIndex] = ' ';  
      bufferIndex++;
      delay(700); 
    }
  }
}

void blinkMorseCode(char code) {
  digitalWrite(ledPin, HIGH); 
  if (code == '.') {
    delay(200);  
  }
  else if (code == '-') {
    delay(600);  
  }
  digitalWrite(ledPin, LOW);  
  delay(200);  
}

void startReading() {
  readingFlag = true;
  attachInterrupt(digitalPinToInterrupt(photoresistorPin), endReading, CHANGE);
}

void endReading() {
  if (readingFlag) {
    int lightValue = analogRead(photoresistorPin);

    if (lightValue > threshold) {
      buffer[bufferIndex] = '.';
      bufferIndex++;
    }
    else {
      buffer[bufferIndex] = '-';
      bufferIndex++;
    }

    readingFlag = false;
    attachInterrupt(digitalPinToInterrupt(ledPin), startReading, CHANGE);
  }
}

void printBuffer() {
  BTSerial.print("Morse Code: ");
  for (int i = 0; i < bufferIndex; i++) {
    BTSerial.print(buffer[i]);
  }
  BTSerial.println();
}

void resetBuffer() {
  memset(buffer, 0, sizeof(buffer));
  bufferIndex = 0;
}

String decodeMorseCode(String morseCode) {
  String decodedMessage;
  String currentCharacter;
  int index = 0;
  while (index < morseCode.length()) {
    if (morseCode[index] == '.') {
      currentCharacter += '.';
      index++;
    } else if (morseCode[index] == '-') {
      currentCharacter += '-';
      index++;
    } else if (morseCode[index] == ' ') {
      if (currentCharacter.length() > 0) {
        char decodedChar = decodeMorseCharacter(currentCharacter);
        if (decodedChar != '\0') {
          decodedMessage += decodedChar;
        }
        currentCharacter = "";
      }
      index++;
    } else {
      index++;
    }
  }

  if (currentCharacter.length() > 0) {
    char decodedChar = decodeMorseCharacter(currentCharacter);
    if (decodedChar != '\0') {
      decodedMessage += decodedChar;
    }
  }

  return decodedMessage;
}

char decodeMorseCharacter(String morseCharacter) {
  for (int i = 0; i < 26; i++) {
    if (morseCharacter.equals(letters[i])) {
      return (char)('A' + i);
    }
  }

  for (int i = 0; i < 10; i++) {
    if (morseCharacter.equals(numbers[i])) {
      return (char)('0' + i);
    }
  }

  return '\0'; 
}

